@@ -8,7 +8,14 @@ import logging
 
 
 class Insight:
-    def __init__(self, jira_url, auth, params=None):
+    def __init__(
+        self,
+        jira_url,
+        auth,
+        params=None,
+        webbrowser_auth=False,
+        jsessionid_for_testing=None,
+    ):
         retries = 3
         backoff_factor = 0.3
         status_forcelist = (500, 502, 504)
@@ -31,10 +38,30 @@ class Insight:
         self.retry_session.auth = self.auth
         self.retry_session.params = params
 
+        # TODO: Find out, why the lib does not extract session cookies
+        if webbrowser_auth:
+            from urllib.parse import urlparse
+
+            url = urlparse(self.jira_url)
+            import browser_cookie3
+
+            cookies = browser_cookie3.firefox(domain_name=url.netloc)
+            self.retry_session.cookies = cookies
+
+        if jsessionid_for_testing:
+            self.retry_session.cookies = requests.cookies.cookiejar_from_dict(
+                {"JSESSIONID": jsessionid_for_testing}
+            )
+
+    @lazy
+    def object_schemas(self):
+        return self.get_object_schemas()
+
+    def get_object_schemas(self):
         api_path = "/objectschema/list"
         object_schemas_json_request = self.do_api_request(api_path)
         object_schemas_json = object_schemas_json_request.get("objectschemas", {})
-        self.object_schemas = {}
+        object_schemas = {}
         logging.info("Loading object schemas")
         object_schemas_count = len(object_schemas_json)
         index = 1
@@ -43,10 +70,11 @@ class Insight:
                 f'Loading object schema {object_schema_json["name"]} ({index}/{object_schemas_count})'
             )
             object_schema_id = object_schema_json["id"]
-            self.object_schemas[object_schema_id] = InsightObjectSchema(
+            object_schemas[object_schema_id] = InsightObjectSchema(
                 self, object_schema_id
             )
             index += 1
+        return object_schemas
 
     def __str__(self):
         return f"Insight: {self.jira_url}"
@@ -101,14 +129,18 @@ class InsightObject:
     def update_object(self, attributes: dict):
         attributes_json = []
         for attribute_id, value in attributes.items():
+            if isinstance(value, list):
+                value_list = [{"value": value_list_item} for value_list_item in value]
+            else:
+                value_list = [{"value": value}]
             entry = {
                 "objectTypeAttributeId": attribute_id,
-                "objectAttributeValues": [{"value": value}]
+                "objectAttributeValues": value_list,
             }
             attributes_json.append(entry)
         request_body = {
-            "objectTypeId": self.object_json['objectType']['id'],
-            "attributes": attributes_json
+            "objectTypeId": self.object_json["objectType"]["id"],
+            "attributes": attributes_json,
         }
         response = self.insight.do_api_request(
             "/object/{}".format(self.id), method="put", json=request_body
@@ -161,6 +193,8 @@ class InsightObjectAttribute:
                 "URL",
                 "Email",
                 "Textarea",
+                "Date",
+                "Date Time",
             ]:
                 return value_json.get("value", None)
             if self.object_type_attribute.attribute_type == "Status":
@@ -171,14 +205,6 @@ class InsightObjectAttribute:
                 return float(value_json.get("value", None))
             if self.object_type_attribute.attribute_type == "Boolean":
                 return value_json.get("value", "false") == "true"
-            if self.object_type_attribute.attribute_type == "Date":
-                return datetime.datetime.strptime(
-                    value_json.get("value", None), "%d.%m.%Y"
-                ).date()
-            if self.object_type_attribute.attribute_type == "Date Time":
-                return datetime.datetime.strptime(
-                    value_json.get("value", None), "%d.%m.%Y %H:%M"
-                )
 
     def __str__(self):
         return f"InsightObjectAttribute: {self.name}, Value: {self.value}"
@@ -193,37 +219,53 @@ class InsightObjectSchema:
         self.key = object_schema_json.get("objectSchemaKey", None)
         self.description = object_schema_json.get("description", None)
 
+    @lazy
+    def object_types(self):
+        return self.get_object_types()
+
+    def get_object_types(self):
         object_types_json = self.insight.do_api_request(
             f"/objectschema/{self.id}/objecttypes/flat"
         )
-        self.object_types = {}
+        object_types = {}
         for object_type in object_types_json:
-            self.object_types[object_type["id"]] = InsightObjectType(
+            object_types[object_type["id"]] = InsightObjectType(
                 self.insight, object_type["id"], object_type
             )
+        return object_types
 
+    @lazy
+    def object_type_attributes(self):
+        return self.get_object_type_attributes()
+
+    def get_object_type_attributes(self):
         object_type_attributes_json = self.insight.do_api_request(
             f"/objectschema/{self.id}/attributes"
         )
-        self.object_type_attributes = {}
+        object_type_attributes = {}
         for object_type_attribute_json in object_type_attributes_json:
-            self.object_type_attributes[
+            object_type_attributes[
                 object_type_attribute_json["id"]
             ] = InsightObjectTypeAttribute(self, object_type_attribute_json)
+        return object_type_attributes
 
     def __str__(self):
         return f"InsightObjectSchema: {self.name} ({self.key})"
 
     def search_iql(self, iql=None):
         api_path = "/iql/objects"
-        params = {"objectSchemaId": self.id, "resultPerPage": 500, "includeTypeAttributes": "true"}
+        params = {
+            "objectSchemaId": self.id,
+            "resultPerPage": 500,
+            "includeTypeAttributes": "true",
+        }
         if iql is not None:
             params["iql"] = iql
         search_request = self.insight.do_api_request(api_path, params=params)
         search_results = search_request
         objects_json: list = search_results["objectEntries"]
         if not objects_json:
-            return []
+            raise StopIteration
 
         # Get additional pages if necessary
         if search_results["pageSize"] > 1:
@@ -235,12 +277,9 @@ class InsightObjectSchema:
                 page = self.insight.do_api_request(api_path, params=params)
                 objects_json += page["objectEntries"]
 
-        objects_result = []
         for json_object in objects_json:
             object_to_add = InsightObject(self.insight, json_object["id"], json_object)
-            objects_result.append(object_to_add)
-
-        return objects_result
+            yield object_to_add
 
     def object_exists(self, object_id):
         return (
@@ -301,7 +340,7 @@ class InsightObjectTypeAttribute:
                 8: "Email",
                 9: "Textarea",
                 10: "Select",
-                11: "IP Address"
+                11: "IP Address",
             },
             1: "Object",
             2: "User",
